(function () {
	'use strict';

	angular
		.module('passover')
		.controller('passoverPaidController', passoverPaidController);

	passoverPaidController.$inject = ['$scope', '$rootScope','passoverService', '$stateParams', '$state', '$ionicHistory', '$timeout', '$ionicPopup', 'purchases'];

	/* @ngInject */
	function passoverPaidController($scope, $rootScope, passoverService, $stateParams, $state, $ionicHistory, $timeout, $ionicPopup, purchases) {

		console.log('passoverPaidController');

    $scope.thisYear = new Date().getFullYear();
    $scope.lsKey    = 'passover_list_' + $scope.thisYear;
    $scope.limit = 20;
		$scope.readMore = false;
		if(typeof $rootScope.categories == 'undefined'){
			$rootScope.categories = [];
		}
		if(typeof $rootScope.quickPickCat == 'undefined'){
			$rootScope.quickPickCat = [];
		}

    purchases.paid().then(function (paid) {
      $scope.paid = paid;
    });

		function gotoPaid(){
			$ionicHistory.nextViewOptions({
		      historyRoot: true,
		      disableBack: true
		    });
			$state.go('app.passover');
		}
		var myPopup, message;

		function buyStark() {
			purchases.purchase()
        .then(function (data) {
			    if(data.owned){
			    	$scope.paid = true;
            gotoPaid();
			    }
		    })
		    .catch(function (err) {
		      $ionicPopup.alert({ title: 'Error', subTitle: err });
		      console.error(err);
		    });
		}

		$scope.restorePurchase = function() {
			purchases.restore().then(function (purchase) {
        if (purchase.owned) {
          message.close();
          $scope.paid = true;
          myPopup = $ionicPopup.show({
            title: 'You have already purchased access to this list. We are restoring your purchase.'
          });
          myPopup.then(function(res) {
            console.log('Tapped!', res);
          });

          $timeout(function() {
            myPopup.close(); //close the popup after 3 seconds for some reason
            gotoPaid();
          }, 3000);
        } else {
          message.close();
          buyStark();
        }
      }).catch(function (err) {
        $ionicPopup.alert({ title: 'Error', subTitle: err });
        console.error(err);
        message.close();
      });
		};

		$scope.buyItem = function() {
			message = $ionicPopup.show({
				title: 'Connecting to the store',
				subTitle: '<span>.</span><span>.</span><span>.</span>',
				cssClass: 'dots'
			});
      $scope.restorePurchase();
		};

        if ($stateParams.products) {
            $scope.products = $stateParams.products;
            $scope.moreLoad = $scope.products.length > $scope.limit;

        } else if ($stateParams.id) {
            $rootScope.$broadcast('loading:show');
            angular.forEach($rootScope.categories, function (l) {
                if (l.name == $stateParams.id) {
                    $scope.products  = l.products;
                    $scope.moreLoad  = $scope.products.length > $scope.limit;
                    $scope.productsf = l.products;
                }
            });

            $rootScope.$broadcast('loading:hide');

        } else {
            $rootScope.$broadcast('loading:show');

            if (!$rootScope.categories.length) {
                if (!$rootScope.networkOffline) {
                    passoverService.getPassover().then(function (data) {
                        if (data.data) {
                            angular.forEach(data.data, function (v) {
                                $rootScope.categories.push(v);
                            });
                            window.localStorage.setItem($scope.lsKey, JSON.stringify($rootScope.categories));
                        }
                    });
                } else {
                    if (!window.localStorage.getItem($scope.lsKey)) {
                        noInternetNotify()
                    } else {
                        $rootScope.categories = JSON.parse(window.localStorage.getItem($scope.lsKey));
                    }
                }
            }

            $scope.$on('network:offline', function () {
                passoverService.cancellRequests();
                if (!window.localStorage.getItem($scope.lsKey)) {
                    noInternetNotify()
                } else {
                    $rootScope.categories = JSON.parse(window.localStorage.getItem($scope.lsKey));
                }
            });

            $scope.moreLoad        = $rootScope.categories.length > $scope.limit;
            $rootScope.categoriesf = $rootScope.categories;
            $rootScope.$broadcast('loading:hide');
        }

        if ($stateParams.search) {
            $scope.search = $stateParams.search;
        }

        function noInternetNotify() {
            $rootScope.notify('No Internet Connection', 'Please try again when you have a good signal');
            $ionicHistory.backView().go();
            $rootScope.$broadcast('loading:hide');
        }

        /*
        |--------------------------------------------------------------------------------------------------------------|
        | Quick Pick data logic                                                                                        |
        |--------------------------------------------------------------------------------------------------------------|
        */
        if (!$rootScope.quickPickCat.length) {

            if (!$rootScope.networkOffline) {
                getQuickPick()

            } else {

                if (!window.localStorage.getItem('quick_pick')) {
                    noInternetNotify()

                } else {
                    getQuickPickFromLocalStorage()
                }
            }
        }

        function getQuickPick() {
            passoverService.getQuickPick().then(function (resp) {
                if (resp.data) {
                    $rootScope.quickPickCat = resp.data;
                    window.localStorage.setItem('quick_pick', JSON.stringify($rootScope.quickPickCat));
                }
            });
        }

        function getQuickPickFromLocalStorage() {
            $rootScope.quickPickCat = JSON.parse(window.localStorage.getItem('quick_pick'))
        }

        $scope.$on('network:offfine', function () {
            passoverService.cancellRequests();
            if (!window.localStorage.getItem('quick_pick')) {
                noInternetNotify()
            } else {
                getQuickPickFromLocalStorage()
            }
        });

        $scope.loadMoreArray = function () {
            if ($scope.categories.length > $scope.limit) {
                console.log($scope.limit);
                $scope.limit += 50;
                $scope.moreLoad = true;
                $scope.$broadcast('scroll.infiniteScrollComplete');
            }
            else
                $scope.moreLoad = false;
        };

        $scope.loadMoreArrayProducts = function () {
            if ($scope.products.length > $scope.limit) {
                $scope.limit += 20;
                $scope.moreLoad = true;
                $scope.$broadcast('scroll.infiniteScrollComplete');
            } else {
                $scope.moreLoad = false;
            }
        };
    }
})();
