(function() {
	'use strict';

	angular
		.module('barebone.vimeo')
		.factory('vimeoService', vimeoService);

	vimeoService.$inject = ['$http', '$q', 'ENV'];

	/* @ngInject */
	function vimeoService($http, $q, _, ENV) {
		var apiUrl = 'https://api.vimeo.com/users/';
		var currentPage = 1;
		var itemsPerPage = 20;

		var videos = [];

		var service = {
			getChannels: getChannels,
			getVideos: getVideos,
			getMoreVideos: getMoreVideos,
			getVideo: getVideo
		};
		return service;

		// ****************************************************************

		function getChannels(){
			return $http.get('https://api.vimeo.com/me/channels').then(function(response){
				console.log(response);
				return response.data;
			});
		}

		function getVideo(videoId) {
			return $http.get('https://api.vimeo.com/videos/' + videoId).then(function(response){
				return response.data;
			});
		}

		function getMoreVideos(channel) {
			return getVideos(channel, currentPage + 1);
		}

		function getVideos(channel, page) {
			currentPage = page || 1;
			console.log('ser', channel);

			var url = 'https://api.vimeo.com'+ channel+'/videos';
			url += '?page=' + currentPage + '&per_page=' + itemsPerPage;

			return $http.get(url)
				.then(function(response) {
					if (currentPage === 1) {
						videos = [];
					}

					angular.forEach(response.data.data, function(item) {
						var id = item.uri.substring(8);

						var image = item.pictures.sizes[0].link;

						videos.push({
							id: id,
							title: item.name,
							description: item.description,
							date: item['created_time'],
							image: image,
							thumb: image,
							videoId: id
						});
					});

					return {
						videos: videos,
						moreItemsAvailable: videos.length < response.data.total
					};
				});
		}
	}
})();
